start_node = {
    "id": "999",
    "type": "custom",
    "position": {"x": 0, "y": 0},
    "data": {
        "name": "Raw Data",
        "value": "Raw Data",
        "color": "pink.400",
        "isInput": True,
        "parameters": {"Dataset": "Test"},
        "node_type": "preprocess",
    },
    "width": 400,
    "height": 76,
    "selected": False,
    "dragging": False,
    "target": "1",
}

node = {
    "id": "",
    "type": "custom",
    "position": {"x": 0, "y": 0},
    "data": {
        "name": "Remove NAN values",
        "value": "Remove NAN values",
        "parameters": {},
        "node_type": "preprocess",
    },
    "width": 400,
    "height": 76,
    "selected": False,
    "dragging": False,
    "source": "",
    "target": "",
}

end_node = {
    "id": "998",
    "type": "custom",
    "position": {"x": 0, "y": 0},
    "data": {
        "name": "Clean Data",
        "value": "Clean Data",
        "color": "green.400",
        "isOutput": True,
        "parameters": {"Dataset": "Test"},
        "node_type": "preprocess",
    },
    "width": 400,
    "height": 76,
    "selected": False,
    "dragging": False,
    "source": "",
}


start_edge = {
    "source": "999",
    "sourceHandle": None,
    "target": "1",
    "targetHandle": None,
    "type": "custom",
    "id": "999-1",
}


edge = {
    "source": "",
    "sourceHandle": None,
    "target": "",
    "targetHandle": None,
    "type": "custom",
    "id": "",
}


def convert_to_graph(json_data):
    # convert test JSON to same format as data graph
    node_arr = [start_node]
    edge_arr = [start_edge]
    count = 1
    for key, value in json_data.items():
        for key2, value2 in json_data[key].items():
            if len(value2) > 0:
                if key2 == "null_values" or key2 == "too_many_zeros":
                    node["data"]["name"] = "Replace NAN values"
                    node["data"]["value"] = "Replace NAN values"
                    node["data"]["parameters"] = {
                        "column": json_data["errors"]["null_values"].append(
                            json_data["warnings"]["too_many_zeros"]
                        )
                    }
                if key2 == "string_features":
                    node["data"]["name"] = "One Hot Encoding"
                    node["data"]["value"] = "One Hot Encoding"
                    node["data"]["parameters"] = {
                        "columns": json_data["errors"]["string_features"]
                    }
                if key2 == "outliers":
                    node["data"]["name"] = "Remove Outliers"
                    node["data"]["value"] = "Remove Outliers"
                    node["data"]["parameters"] = {
                        "columns": json_data["warnings"]["outliers"]
                    }
                if key2 == "data_normalization":
                    node["data"]["name"] = "Data normalization"
                    node["data"]["value"] = "Data normalization"
                    node["data"]["parameters"] = {
                        "column": json_data["warnings"]["data_normalization"]
                    }

                if count == 1:
                    node["id"] = str(count)
                    node["source"] = "999"
                else:
                    node["id"] = str(count)
                    node["source"] = str(count - 1)
                node["target"] = str(count + 1)
                node_arr.append(node)

                edge["source"] = str(count)
                edge["id"] = str(count) + "-" + str(count + 1)
                edge["target"] = str(count + 1)
                edge_arr.append(edge)

                count += 1
        node_arr.append(end_node)
        main_node = {"nodes": node_arr, "edges": edge_arr}
        return main_node



import pandas as pd
import requests
from pydantic import BaseModel
#import woodwork as ww
from ..util.data_types import get_dataframe_types

class Dataset:
    def __init__(self, url) -> None:
        self.url = url
        self.dataframe = self.__load_dataset(url)

    def __load_dataset(self, url: str) -> pd.DataFrame:
        local_filename = url.split("/")[-1]
        with requests.get(url, stream=True) as r:
            r.raise_for_status()
            with open(local_filename, "wb") as f:
                for chunk in r.iter_content(chunk_size=8192):
                    f.write(chunk)
        return pd.read_csv(local_filename)


    def parse_dataset(self):
        data_dict = get_dataframe_types(self.dataframe)
        data_dict['rows'] = len(self.dataframe.axes[0])
        return data_dict





import requests
from .experiment import Experiment

class Deployment:

    def generate_code(self, experiment_id: str):
        experiment = Experiment(experiment_id=experiment_id)
        with open("deployment.py", "w") as f:
            f.write("import requests")
            f.write("\nimport os")
            f.write("\nimport pandas as pd")
            f.write("\nfrom pydantic import BaseModel")
            f.write("\nfrom huble import Experiment")
            f.write("\nfrom typing import Union")
            f.write("\nimport joblib")
            f.write("\nexperiment_id = os.getenv(\"EXPERIMENT_ID\")")
            f.write(f"\nexperiment = Experiment(experiment_id)")
            f.write("\n")
            f.write("\nclass RequestBody(BaseModel):")
            for column in experiment.input_format["columns"]:
              if(column['name'] != "index"):
                f.write(f"\n\t{column['name']}: Union[int, float]")
            f.write("\n")
            f.write(f"\nmodel_url = experiment.modelURL")
            f.write('\nmodel_content = requests.get(f"https://ipfs.filebase.io/ipfs/{model_url}")')
            f.write("\nopen('model.joblib', 'wb').write(model_content.content)")
            f.write("\nmodel = joblib.load('model.joblib')")
            f.write("\n")
            f.write("\n")
            f.write("\ndef predict(request_body: RequestBody):")
            f.write("\n\trequest_body = request_body.dict()")
            f.write("\n\tX = pd.DataFrame(request_body, index=[0])")
            f.write("\n\treturn model.predict(X)")






import pandas as pd


def get_dataframe_types(dataframe):
    data_dict = pd.io.json.build_table_schema(dataframe)  # type: ignore
    for field in data_dict["fields"]:
        if field["type"] == "integer":
            field["type"] = "number"
        if field["type"] == "any":
            field.pop("constraints")
            field.pop("ordered")
            field["type"] = "categorical"
    data_dict["columns"] = data_dict.pop("fields")
    data_dict.pop("primaryKey")
    data_dict.pop("pandas_version")
    return data_dict
